# -*- coding: utf-8 -*-

from PyQt4.QtCore import *
from PyQt4.QtGui import QImage, QPainter, QCursor
from PyKDE4.plasma import Plasma
from PyKDE4.kdeui import KApplication, KWindowSystem
from PyKDE4 import plasmascript
import random

HorizontalUp, VerticalRight, HorizontalDown, VerticalLeft = range(4)
Moving, Waiting, BlinkingPre, BlinkingAfter = range(4)

SUNBASE = "SunBase"
SUNSMILE = "SunSmile"
SUNBLINK = "SunBlink"

MAXTIME = 90
ANIMATIONSPEED = 50

class BSun(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)

    def init(self):
        self.setAspectRatioMode(Plasma.KeepAspectRatio)

        self.setHasConfigurationInterface(False)
        self.setBackgroundHints(Plasma.Applet.NoBackground)

        self.orientation = HorizontalUp

        self.rotation = 0
        self.animationState = Moving
        self.animationChangeTime = self.getRandomMaxTime()
        self.animationTimePassed = 0

        self.svgSun = Plasma.Svg(self)
        self.svgSun.setImagePath(self.package().filePath("images", "bsun.svg"))

        self.screenSize = self.screenSize()
        self.setMaximumSize(QSizeF(self.screenSize.width() / 5.0, self.screenSize.width() / 5.0 * self.aspectRatio()))
        self.resize(QSizeF(self.maximumWidth()/2.0, self.maximumHeight()/2.0))
        self.updateSvgSize(self.size())

        self.mousePressed = False

        self.timer = QTimer(self)
        self.connect(self.timer, SIGNAL("timeout ()"), self.animation)
        self.timer.start(ANIMATIONSPEED)

    def screenSize(self):
        app = KApplication.kApplication()
        if app.desktop().screenCount() > 1 or app.desktop().isVirtualDesktop():
            screenNum = KApplication.desktop().screenNumber(QCursor.pos()) #unreliable
            return KWindowSystem.workArea().intersect(app.desktop().screenGeometry(screenNum))
        else:
            return KWindowSystem.workArea()

    def getRandomMaxTime(self):
        return (random.random() * MAXTIME)

    def aspectRatio(self):
        return (300.0/300.0)
 
    def paintInterface(self, painter, option, rect):
        painter.setRenderHints(QPainter.SmoothPixmapTransform)
        painter.save()
        translateWidth = self.geometry().width() * 1/2
        translateheight = self.geometry().height() * 1/2
        painter.translate(translateWidth, translateheight)
        painter.rotate(self.rotation)
        painter.translate(-translateWidth, -translateheight)
        self.svgSun.paint(painter, QPointF(0,0), SUNBASE)
        painter.restore()

        if (self.animationState == Waiting) or (self.animationState == Moving) or (self.animationState == BlinkingAfter):
            elementSize = self.svgSun.elementSize(SUNSMILE)
            painter.translate(translateWidth - elementSize.width() * 1/2, translateheight - elementSize.height() * 1/2)
            self.svgSun.paint(painter, QPointF(0, 0), SUNSMILE)
        elif self.animationState == BlinkingPre:
            elementSize = self.svgSun.elementSize(SUNSMILE)
            painter.translate(translateWidth - elementSize.width() * 1/2, translateheight - elementSize.height() * 1/2 + elementSize.height()/4)
            self.svgSun.paint(painter, QPointF(0, 0), SUNBLINK)

    def updateSvgSize(self, size):
        self.svgSun.resize(size)

    def animation(self):
        if self.mousePressed:
            return

        self.animationTimePassed += ANIMATIONSPEED/1000.0

        #print(str(self.animationState) + " " + str(self.rotation) + " " + str(Moving))
        if self.animationState == Moving:
            #50.0/1000.0 * 360.0/2.0 = 9 / 2 = 4.5
            self.rotation += 2
            if self.rotation > 360:
                self.rotation -= 360

            #50/(1000/300) == 15 / 2 = 7.5
            pixDiff = 5

            newX = self.x()
            newY = self.y()

            if self.orientation == HorizontalUp:
                #print("HorizontalUp")
                newX += pixDiff
                if newX > self.screenSize.x() + self.screenSize.width() - self.geometry().width():
                    newX = self.screenSize.x() + self.screenSize.width() - self.geometry().width()
                    #newY += self.geometry().width()
                    self.orientation = VerticalRight
            elif self.orientation == VerticalRight:
                #print("VerticalRight")
                newY -= pixDiff
                if newY < self.screenSize.y():
                    newY = self.screenSize.y()
                    #newX += self.geometry().height()
                    self.orientation = HorizontalDown
            elif self.orientation == HorizontalDown:
                #print("HorizontalDown")
                newX -= pixDiff
                if newX < self.screenSize.x():
                    newX = self.screenSize.x()
                    #newY -= self.geometry().width()
                    self.orientation = VerticalLeft
            elif self.orientation == VerticalLeft:
                #print("VerticalLeft")
                newY += pixDiff
                if newY > self.screenSize.y() + self.screenSize.height() - self.geometry().height():
                    newY = self.screenSize.y() + self.screenSize.height() - self.geometry().height()
                    #newX -= self.geometry().width()
                    self.orientation = HorizontalUp
            self.setPos(newX, newY)
            self.update()

        if self.animationTimePassed > self.animationChangeTime:
            self.animationTimePassed = 0
            print("CHANGE")
            if self.animationState == Moving:
                self.animationState = Waiting
                self.animationChangeTime = self.getRandomMaxTime() / 8
                self.update()
            elif self.animationState == Waiting:
                self.animationState = BlinkingPre
                self.animationChangeTime = 2
                self.update()
            elif self.animationState == BlinkingPre:
                self.animationState = BlinkingAfter
                self.animationChangeTime = self.getRandomMaxTime() / 8
                self.update()
            elif self.animationState == BlinkingAfter:
                self.animationState = Moving
                self.animationChangeTime = self.getRandomMaxTime()
            print(self.animationChangeTime)
            print("STATE: " + str(self.animationState))

    def constraintsEvent(self, constraints):
        if constraints & Plasma.FormFactorConstraint:
            self.setBackgroundHints(Plasma.Applet.NoBackground)

        if constraints & Plasma.SizeConstraint:
            self.updateSvgSize(self.size())

    def mouseReleaseEvent(self, event):
        print("release")
        self.mousePressed = False;
        event.accept()

    def mousePressEvent(self, event):
        print("press")
        self.mousePressed = True;
        event.accept()


def CreateApplet(parent):
    return BSun(parent)

